@@ -1,9 +1,6 @@
-#' CLV Model functionality for PNBD with static covariates
+#' @templateVar name_model_full Pareto/NBD
+#' @template template_class_clvmodelstaticcov
 #'
-#' This class implements the functionalities and model-specific steps which are required
-#' to fit the Pareto/NBD model with static covariates.
-#'
-#' @keywords internal
 #' @seealso Other clv model classes \link{clv.model-class}, \link{clv.model.pnbd.no.cov-class}, \link{clv.model.pnbd.dynamic.cov-class}
 #' @seealso Classes using its instance: \link{clv.fitted.static.cov-class},
 #'
@@ -14,7 +11,7 @@ setClass(Class = "clv.model.pnbd.static.cov", contains = "clv.model.pnbd.no.cov"
 
          # Prototype is labeled not useful anymore, but still recommended by Hadley / Bioc
          prototype = list(
-           start.param.cov = numeric()))
+           start.param.cov = numeric(0)))
 
 
 #' @importFrom methods new
@@ -51,7 +48,7 @@ setMethod(f = "clv.model.check.input.args", signature = signature(clv.model="clv
 
               # Nothing model-specific to check about all other user inputs
               # Nothing to return
-})
+            })
 
 # . clv.model.put.estimation.input ------------------------------------------------------------------------------------------------------------
 #   Use pnbd.no.cov methods, dont need to overwrite
@@ -92,9 +89,12 @@ setMethod(f = "clv.model.prepare.optimx.args", signature = signature(clv.model="
                                         vX      = clv.fitted@cbs$x,
                                         vT_x    = clv.fitted@cbs$t.x,
                                         vT_cal  = clv.fitted@cbs$T.cal,
-                                        # Covariate data, as matrix!
-                                        mCov_life  = clv.data.get.matrix.data.cov.life(clv.fitted@clv.data),
-                                        mCov_trans = clv.data.get.matrix.data.cov.trans(clv.fitted@clv.data)))
+
+                                        mCov_life  = clv.data.get.matrix.data.cov.life(clv.data = clv.fitted@clv.data, correct.row.names=clv.fitted@cbs$Id,
+                                                                                       correct.col.names=clv.data.get.names.cov.life(clv.fitted@clv.data)),
+                                        mCov_trans = clv.data.get.matrix.data.cov.trans(clv.data = clv.fitted@clv.data, correct.row.names=clv.fitted@cbs$Id,
+                                                                                        correct.col.names=clv.data.get.names.cov.trans(clv.fitted@clv.data))),
+                                      keep.null = TRUE)
             return(optimx.args)
           })
 
@@ -125,27 +125,21 @@ setMethod("clv.model.predict.clv", signature(clv.model="clv.model.pnbd.static.co
   # cran silence
   period.length <- CET <- i.CET <- x <- t.x <- T.cal <- PAlive <- i.PAlive <- i.DERT <- DERT <- NULL
 
-  # Covariates as matrix, if there is a covariate
-  data.cov.mat.life  <- clv.data.get.matrix.data.cov.life(clv.fitted@clv.data)
-  data.cov.mat.trans <- clv.data.get.matrix.data.cov.trans(clv.fitted@clv.data)
-
-  # **TODO:Check that matrices have same order as cbs?? (stopifnot(all(rownames() == cbs$Id))?)
-
-  # Read out from table
   predict.number.of.periods <- dt.prediction[1, period.length]
 
-  # Put params together in single vec
-  estimated.params <- c(r = clv.fitted@prediction.params.model[["r"]], alpha = clv.fitted@prediction.params.model[["alpha"]],
-                        s = clv.fitted@prediction.params.model[["s"]], beta  = clv.fitted@prediction.params.model[["beta"]])
-
-
   # To ensure sorting, do everything in a single table
   dt.result <- copy(clv.fitted@cbs[, c("Id", "x", "t.x", "T.cal")])
-
+  data.cov.mat.life  <- clv.data.get.matrix.data.cov.life(clv.data = clv.fitted@clv.data, correct.row.names=dt.result$Id,
+                                                          correct.col.names=names(clv.fitted@prediction.params.life))
+  data.cov.mat.trans <- clv.data.get.matrix.data.cov.trans(clv.data = clv.fitted@clv.data, correct.row.names=dt.result$Id,
+                                                           correct.col.names=names(clv.fitted@prediction.params.trans))
 
   # Add CET
-  dt.result[, CET :=  pnbd_staticcov_CET(vEstimated_params  = estimated.params,
-                                         dPrediction_period = predict.number.of.periods,
+  dt.result[, CET :=  pnbd_staticcov_CET(r       = clv.fitted@prediction.params.model[["r"]],
+                                         alpha_0 = clv.fitted@prediction.params.model[["alpha"]],
+                                         s       = clv.fitted@prediction.params.model[["s"]],
+                                         beta_0  = clv.fitted@prediction.params.model[["beta"]],
+                                         dPeriods = predict.number.of.periods,
                                          vX     = x,
                                          vT_x   = t.x,
                                          vT_cal = T.cal,
@@ -155,7 +149,10 @@ setMethod("clv.model.predict.clv", signature(clv.model="clv.model.pnbd.static.co
                                          mCov_life   = data.cov.mat.life)]
 
   # Add PAlive
-  dt.result[, PAlive := pnbd_staticcov_PAlive(vEstimated_params = estimated.params,
+  dt.result[, PAlive := pnbd_staticcov_PAlive(r       = clv.fitted@prediction.params.model[["r"]],
+                                              alpha_0 = clv.fitted@prediction.params.model[["alpha"]],
+                                              s       = clv.fitted@prediction.params.model[["s"]],
+                                              beta_0  = clv.fitted@prediction.params.model[["beta"]],
                                               vX     = x,
                                               vT_x   = t.x,
                                               vT_cal = T.cal,
@@ -165,7 +162,10 @@ setMethod("clv.model.predict.clv", signature(clv.model="clv.model.pnbd.static.co
                                               mCov_life  = data.cov.mat.life)]
 
   # Add DERT
-  dt.result[, DERT := pnbd_staticcov_DERT(vEstimated_params = estimated.params,
+  dt.result[, DERT := pnbd_staticcov_DERT(r       = clv.fitted@prediction.params.model[["r"]],
+                                          alpha_0 = clv.fitted@prediction.params.model[["alpha"]],
+                                          s       = clv.fitted@prediction.params.model[["s"]],
+                                          beta_0  = clv.fitted@prediction.params.model[["beta"]],
                                           continuous_discount_factor = continuous.discount.factor,
                                           vX     = x,
                                           vT_x   = t.x,
@@ -191,14 +191,18 @@ setMethod("clv.model.expectation", signature(clv.model="clv.model.pnbd.static.co
 
   #calculate alpha_i, beta_i
   params_i <- clv.fitted@cbs[, c("Id", "T.cal", "date.first.actual.trans")]
+  m.cov.data.life  <- clv.data.get.matrix.data.cov.life(clv.data=clv.fitted@clv.data, correct.row.names=params_i$Id,
+                                                        correct.col.names=names(clv.fitted@prediction.params.life))
+  m.cov.data.trans <- clv.data.get.matrix.data.cov.trans(clv.data=clv.fitted@clv.data, correct.row.names=params_i$Id,
+                                                         correct.col.names=names(clv.fitted@prediction.params.trans))
 
   # all params exactly the same for all customers as there are no covariates
   params_i[, r       := clv.fitted@prediction.params.model[["r"]]]
   params_i[, s       := clv.fitted@prediction.params.model[["s"]]]
 
   # Alpha is for trans, beta for live!
-  params_i[, alpha_i := clv.fitted@prediction.params.model[["alpha"]] * exp( -clv.data.get.matrix.data.cov.trans(clv.fitted@clv.data) %*% clv.fitted@prediction.params.trans)]
-  params_i[, beta_i  := clv.fitted@prediction.params.model[["beta"]]  * exp( -clv.data.get.matrix.data.cov.life(clv.fitted@clv.data)  %*% clv.fitted@prediction.params.life)]
+  params_i[, alpha_i := clv.fitted@prediction.params.model[["alpha"]] * exp( -m.cov.data.trans %*% clv.fitted@prediction.params.trans)]
+  params_i[, beta_i  := clv.fitted@prediction.params.model[["beta"]]  * exp( -m.cov.data.life  %*% clv.fitted@prediction.params.life)]
 
 
   # To caluclate expectation at point t for customers alive in t, given in params_i.t
